## ############################################################################
##
## The 2022 Europe Report of the Lancet Countdown on Health and Climate Change:
##                 towards a climate resilient future
##
## Indicator 1.2.2: Drought Indicator
##
## ############################################################################
##
## Authors: 
## - Balakrishnan Solaraju Murali, ORCID: 0000-0003-0013-3295
## - Nube González Reviriego, ORCID: 0000-0002-5919-6701
##
## Contributor:
## - Prof. Rachel Lowe, ORCID: 0000-0003-3939-7343
##
## Contact information: balakrishnan.solaraju@bsc.es
## 
## ############################################################################

# Atomic functions to support PET and SPEI estimation

# # Atomic function for PET estimation

atomic_pet <- function(data_tasmax, data_tasmin, pet_method, lat_mask){
  
  source("/esarchive/scratch/bsolaraj/LANCET/functions/Support_PET_estimation.R")
  
  ftime=as.numeric(dim(data_tasmax)['time'])
  n_sdate= as.numeric(dim(data_tasmax)['sdate'])
  lat <- lat_mask
  
  pet_estd <- PETx(tmean_mod = NA, pre_mod = NA, tmax_mod=data_tasmax, tmin_mod=data_tasmin, pet_method=pet_method, lat=lat, na.rm = TRUE)
  return(pet_estd)
  
}

# Atomic function for P-PET

atomic_accum <- function(data, accum_scale){
  
  ftime=as.numeric(dim(data)['time'])
  n_sdate= as.numeric(dim(data)['sdate'])
  scale <- accum_scale
  
  # Accumulation at different timescales
  data_vector <- as.vector(data)
  data_sum_x <- rollapply(data_vector, scale, sum)
  data_sum <- c(rep(NA, scale-1),data_sum_x)
  Accum6 <- array(data_sum, dim = c(ftime,n_sdate))
  
  return(Accum6)
  
}

# Atomic function for SPEI estimation

atomic_spei <- function(data){
  
  source("/esarchive/scratch/bsolaraj/LANCET/functions/Support_SPEIX.R")
  source("/esarchive/scratch/bsolaraj/LANCET/functions/Support_SPEIX_handler.R")
  
  n_sdate= as.numeric(dim(data)['sdate'])
  
  speiX <- spX_crossvalid(data, ref.start=NULL, ref.end=NULL, method ='parametric', distribution = 'log-Logistic', fit='ub-pwm')
  
  return(speiX)
  
}
